/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.SearchBean;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.AbstractShrimpTool;

/**
 * @author Rob Lintern
 */
public class SearchTool extends AbstractShrimpTool {
	
    /**
     * A tabbed pane is used to hold the component UI of each search strategy.
     */
    private JTabbedPane strategiesTabbedPane = null;
    private JRootPane gui;
    private ShrimpViewSearchToolBridge bridge;

    public SearchTool(ShrimpProject project) {
        super(ShrimpProject.SEARCH_TOOL, project);

        gui = new JRootPane();
        gui.setLayout(new BorderLayout());
        bridge = new ShrimpViewSearchToolBridge(this);
    }

    /**
     * @see ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool#getGUI()
     */
    public Component getGUI() {
        return gui;
    }

    /**
     * @see ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool#disposeTool()
     */
    public void disposeTool() {
    	bridge.removeListeners();
    	clear();
    }

    /**
     * @see ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool#refresh()
     */
    public void refresh() {
        if (project == null)
            return;
        
        gui.removeAll();
        try {
            SearchBean searchBean = (SearchBean) project.getBean(ShrimpProject.SEARCH_BEAN);
            searchBean.addSearchBeanListener(new SearchBeanListener() {
                 public void strategiesChanged() {
                     refresh();
                 }
            });
            JPanel pnlAllStrategies = new JPanel (new BorderLayout ());
            strategiesTabbedPane = new JTabbedPane();
            strategiesTabbedPane.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    selectedStrategyChanged ();
                }
            });
            strategiesTabbedPane.removeAll();
            for (Iterator iter = searchBean.getStrategies().iterator(); iter.hasNext();) {
                SearchStrategy searchStrategy = (SearchStrategy) iter.next();
                strategiesTabbedPane.addTab(searchStrategy.getName(), searchStrategy.getComponentUI());
            }
            pnlAllStrategies.add(strategiesTabbedPane, BorderLayout.CENTER);
            gui.add(new JScrollPane(pnlAllStrategies), BorderLayout.CENTER);
            gui.revalidate();
        } catch (BeanNotFoundException e) {
            // do nothing
        }
        
        bridge.init();
    }
    
    public void setSearchText(String text) {
		try {
			SearchBean searchBean = (SearchBean) project.getBean(ShrimpProject.SEARCH_BEAN);
	        for (Iterator iter = searchBean.getStrategies().iterator(); iter.hasNext();) {
	            SearchStrategy searchStrategy = (SearchStrategy) iter.next();
	            searchStrategy.setSearchText(text);
	        }
		} catch (BeanNotFoundException e) {
		}
    }
    
    private void selectedStrategyChanged() {
        if (project == null)
            return;

        try {
            SearchBean searchBean = (SearchBean) project.getBean(ShrimpProject.SEARCH_BEAN);
	        Component selectedStrategyGUI = strategiesTabbedPane.getSelectedComponent();
	        for (Iterator iter = searchBean.getStrategies().iterator(); iter.hasNext();) {
	            final SearchStrategy strategy = (SearchStrategy) iter.next();
	            if (strategy.getComponentUI().equals(selectedStrategyGUI)) {
	                SwingUtilities.invokeLater(new Runnable() {
	                    public void run() {
	                        setSearchButtonAsDefault(strategy);
	                    }
	                });
	                break;
	            }
	        }
        } catch (BeanNotFoundException e) {
            // do nothing
        }
   }

    private void setSearchButtonAsDefault(SearchStrategy selectedStrategy) {
        JButton btnSearch = selectedStrategy.getSearchButton();
        if (btnSearch != null) {
            gui.setDefaultButton(btnSearch);
        }
    }

    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool#clear()
     */
    public void clear() {
    	gui.removeAll();
    }

}